@@ -112,8 +112,6 @@ PGAPI_Connect(
 	memcpy(&ci->drivers, &globals, sizeof(globals));
 	getDSNinfo(ci, CONN_OVERWRITE);
 	logs_on_off(1, ci->drivers.debug, ci->drivers.commlog);
-	/* initialize pg_version from connInfo.protocol    */
-	CC_initialize_pg_version(conn);
 
 	/*
 	 * override values from DSN info with UID and authStr(pwd) This only
@@ -740,36 +738,6 @@ CC_lookup_lo(ConnectionClass *self)
 }
 
 
-/*
- *	This function initializes the version of PostgreSQL from
- *	connInfo.protocol that we're connected to.
- *	h-inoue 01-2-2001
- */
-void
-CC_initialize_pg_version(ConnectionClass *self)
-{
-	strcpy(self->pg_version, self->connInfo.protocol);
-	if (PROTOCOL_62(&self->connInfo))
-	{
-		self->pg_version_number = (float) 6.2;
-		self->pg_version_major = 6;
-		self->pg_version_minor = 2;
-	}
-	else if (PROTOCOL_63(&self->connInfo))
-	{
-		self->pg_version_number = (float) 6.3;
-		self->pg_version_major = 6;
-		self->pg_version_minor = 3;
-	}
-	else
-	{
-		self->pg_version_number = (float) 6.4;
-		self->pg_version_major = 6;
-		self->pg_version_minor = 4;
-	}
-}
-
-
 /*
  *	This function gets the version of PostgreSQL that we're connected to.
  *	This is used to return the correct info in SQLGetInfo
@@ -1343,6 +1311,9 @@ CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 		ci->updatable_cursors = ci->allow_keyset;
 #endif /* DRIVER_CURSOR_IMPLEMENT */
 
+	if (!CC_is_in_autocommit(self))
+		CC_commit(self);
+
 	CC_clear_error(self);		/* clear any initial command errors */
 	self->status = CONN_CONNECTED;
 
@@ -1423,21 +1394,14 @@ QResultClass *
 CC_send_query(ConnectionClass *self, char *query, QueryInfo *qi, UDWORD flag)
 {
 	QResultClass *cmdres = NULL,
-			   *retres = NULL,
-			   *res ;
+			     *retres = NULL,
+			     *res ;
 	BOOL	clear_result_on_abort = ((flag & CLEAR_RESULT_ON_ABORT) != 0),
-		create_keyset = ((flag & CREATE_KEYSET) != 0),
-		issue_begin = ((flag & GO_INTO_TRANSACTION) != 0 && !CC_is_in_trans(self));
-	char		 *wq;
-	int			id=0;
+			create_keyset = ((flag & CREATE_KEYSET) != 0),
+			issue_begin = ((flag & GO_INTO_TRANSACTION) != 0 && !CC_is_in_trans(self));
 	PGconn *pgconn = self->pgconn;
-	int			maxlen,
-				empty_reqs;
-	BOOL		ReadyToReturn = FALSE,
-				query_completed = FALSE,
-				before_64 = PG_VERSION_LT(self, 6.4),
-				aborted = FALSE,
-				used_passed_result_object = FALSE;
+	int			maxlen;
+	BOOL		used_passed_result_object = FALSE;
 	int		func_cs_count = 0;
 
 
@@ -1467,104 +1431,110 @@ CC_send_query(ConnectionClass *self, char *query, QueryInfo *qi, UDWORD flag)
 	if (issue_begin)
     {
 		res = LIBPQ_execute_query(self,"BEGIN");
-        QR_Destructor(res);
+		if ((!res) || (res->status != PGRES_COMMAND_OK))
+		{
+			CC_set_error(self, CONNECTION_COULD_NOT_SEND, "Could not send Query to backend");
+			CC_on_abort(self, NO_TRANS | CONN_DEAD);
+			QR_Destructor(res);
+			res = NULL;
+			goto cleanup;
+		}
     }
     
 	res = LIBPQ_execute_query(self,query);
 
-	if((!res) || (res->status == PGRES_EMPTY_QUERY) )
+	if(!res)
 	{
+		goto cleanup;
+	}
+#ifdef NOT_USED
+	else if ((res->status == PGRES_EMPTY_QUERY) || (res->status == PGRES_FATAL_ERROR) ||
+		     (res->status == PGRES_BAD_RESPONSE))
+	{
+		mylog("send_query: failed -> abort\n");
+		QR_set_aborted(res, TRUE);
 		QR_Destructor(res);
         res = NULL;
 		goto cleanup;
 	}
+#endif /* NOT_USED */
+	else if  (res->status == PGRES_COMMAND_OK)
+	{
+		mylog("send_query: done sending command\n");
+		goto cleanup;
+	}
 	else
 	{
-		mylog("send_query: done sending query\n");
+		mylog("send_query: done sending query with status: %i\n",res->status);
 
-		empty_reqs = 0;
-		for (wq = query; isspace((UCHAR) *wq); wq++)
-		;
-		if (*wq == '\0')
-			empty_reqs = 1;
 		cmdres = qi ? qi->result_in : NULL;
 		if (cmdres)
 			used_passed_result_object = TRUE;
 		if (!used_passed_result_object)
 		{
+			if ((res->status == PGRES_EMPTY_QUERY) || (res->status == PGRES_BAD_RESPONSE))
+			{
+				mylog("send_query: sending query failed -> abort\n");
+				QR_set_aborted(res, TRUE);
+				QR_Destructor(res);
+				res = NULL;
+				goto cleanup;
+			}
+			else if (res->status == PGRES_FATAL_ERROR)
+			{
+				mylog("send_query: sended query failed -> abort\n");
+				QR_set_aborted(res, TRUE);
+				goto cleanup;
+			}
 			if (create_keyset)
 				QR_set_haskeyset(res->next);
 			if (!QR_fetch_tuples(res, self, qi ? qi->cursor : NULL))
 			{
 				CC_set_error(self, CONNECTION_COULD_NOT_RECEIVE, QR_get_message(res));
-				if (PGRES_FATAL_ERROR == QR_get_status(res))
-					retres = cmdres;
-				else
-					retres = NULL;
 			}
 		}
 		else
-		{				/* next fetch, so reuse an existing result */
-			/*
-			* called from QR_next_tuple and must return
-			* immediately.
-			*/
+		{
+			/* next fetch, so reuse an existing result */
+			mylog("send_query: next fetch -> reuse result\n");
 			if (!QR_fetch_tuples(res, NULL, NULL))
 			{
 				CC_set_error(self, CONNECTION_COULD_NOT_RECEIVE, QR_get_message(res));
-				retres = NULL;
+				if (PGRES_FATAL_ERROR == QR_get_status(res))
+				{
+					QR_set_aborted(res, TRUE);
+					retres = cmdres;
+				}
 			}
-			retres = cmdres;
 		}
 	}
 
 cleanup:
 	CLEANUP_FUNC_CONN_CS(func_cs_count, self);
-	/*
-	 * Cleanup garbage results before returning.
-	 */
-	if (cmdres && retres != cmdres && !used_passed_result_object)
-	{
-		QR_Destructor(cmdres);
-	}
+#undef	return
 	/*
 	 * Cleanup the aborted result if specified
 	 */
 	if (retres)
 	{
-		if (aborted)
+		/*
+		 *	discard results other than errors.
+		 */
+		QResultClass	*qres;
+		for (qres = retres; qres->next; qres = retres)
 		{
-			if (clear_result_on_abort)
-			{
-	   			if (!used_passed_result_object)
-				{
-					QR_Destructor(retres);
-					retres = NULL;
-				}
-			}
-			if (retres)
-			{
-				/*
-				 *	discard results other than errors.
-				 */
-				QResultClass	*qres;
-				for (qres = retres; qres->next; qres = retres)
-				{
-					if (QR_get_aborted(qres))
-						break;
-					retres = qres->next;
-					qres->next = NULL;
-					QR_Destructor(qres);
-				}
-				/*
-				 *	If error message isn't set
-				 */
-				if (retres && (!CC_get_errormsg(self) || !CC_get_errormsg(self)[0]))
-					CC_set_errormsg(self, QR_get_message(retres));
-			}
+			if (QR_get_aborted(qres))
+				break;
+			retres = qres->next;
+			qres->next = NULL;
+			QR_Destructor(qres);
 		}
+		/*
+		 *	If error message isn't set
+		 */
+		if (retres && (!CC_get_errormsg(self) || !CC_get_errormsg(self)[0]))
+			CC_set_errormsg(self, QR_get_message(retres));
 	}
-#undef	return
 	return res;
 }
 
@@ -1604,8 +1574,8 @@ CC_log_error(const char *func, const char *desc, const ConnectionClass *self)
 
 	if (self)
 	{
-		qlog("CONN ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__error_message));
-		mylog("CONN ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__error_message));
+		qlog("CONN ERROR: func=%s, desc='%s', errnum=%d, sqlstate=%s, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__sqlstate), nullcheck(self->__error_message));
+		mylog("CONN ERROR: func=%s, desc='%s', errnum=%d, sqlstate=%s, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__sqlstate), nullcheck(self->__error_message));
 		qlog("            ------------------------------------------------------------\n");
 		qlog("            henv=%u, conn=%u, status=%u, num_stmts=%d\n", self->henv, self, self->status, self->num_stmts);
 	}
@@ -1777,21 +1747,38 @@ QResultClass *
 LIBPQ_execute_query(ConnectionClass *self,char *query)
 {
 	QResultClass	*qres;
-	PGresult	*pgres;
+	PGresult	*pgres = NULL, *pgresnew = NULL;
 	char		errbuffer[ERROR_MSG_LENGTH + 1];
 	int		pos=0;
 
+	mylog("LIBPQ_execute_query: entering ...\n");
 	qres=QR_Constructor();
 	if(!qres)
 	{
 		CC_set_error(self, CONNECTION_COULD_NOT_RECEIVE, "Could not allocate memory for result set");
+		CC_on_abort(self, CONN_DEAD);
 		QR_Destructor(qres);
-        	return NULL;
+		return NULL;
 	}
 
-        PQsetNoticeProcessor(self->pgconn, CC_handle_notice, qres);
-        pgres = PQexec(self->pgconn,query);
-        PQsetNoticeProcessor(self->pgconn, CC_handle_notice, NULL);
+	PQsetNoticeProcessor(self->pgconn, CC_handle_notice, qres);
+	if (!PQsendQuery(self->pgconn,query))
+	{
+		CC_set_error(self, CONNECTION_COULD_NOT_RECEIVE, "Could not send query to backend");
+		CC_on_abort(self, CONN_DEAD);
+		QR_Destructor(qres);
+		return NULL;
+	}
+	while (pgresnew = PQgetResult(self->pgconn))
+	{
+		mylog("LIBPQ_execute_query: get next result with status = %i\n",PQresultStatus(pgresnew));
+		if (pgres)
+			PQclear(pgres);
+		pgres = pgresnew;
+	}
+	PQsetNoticeProcessor(self->pgconn, CC_handle_notice, NULL);
+
+	mylog("LIBPQ_execute_query: query = %s\n",query);
 
 	qres->status = PQresultStatus(pgres);
 	
@@ -1811,30 +1798,33 @@ LIBPQ_execute_query(ConnectionClass *self,char *query)
 		return qres;
 	}
     
-	if (strnicmp(query, "BEGIN", 5) == 0)
-	{
-		CC_set_in_trans(self);
-	}
-	else if (strnicmp(query, "COMMIT", 6) == 0)
-		CC_on_commit(self);
-	else if (strnicmp(query, "ROLLBACK", 8) == 0)
-	{
-		/* 
-		   The method of ROLLBACK an original form ....
-		   ROLLBACK [ WORK | TRANSACTION ] TO [ SAVEPOINT ] savepoint_name
-		 */
-		if (PG_VERSION_LT(self, 8.0) || !(contains_token(query, " TO ")))
-			CC_on_abort(self, NO_TRANS);
-	}
-	else if (strnicmp(query, "END", 3) == 0)
-		CC_on_commit(self);
-	else if (strnicmp(query, "ABORT", 5) == 0)
-		CC_on_abort(self, NO_TRANS);
-	else
-		qres->recent_processed_row_count = atoi(PQcmdTuples(pgres));
-
 	if( (PQresultStatus(pgres) == PGRES_COMMAND_OK) )
 	{
+		if ((strnicmp(query, "BEGIN", 5) == 0) ||
+		    (strnicmp(query, "START TRANSACTION", 17) == 0))
+			CC_set_in_trans(self);
+		else if ((strnicmp(query, "COMMIT", 6) == 0) ||
+		         (strnicmp(query, "END", 3) == 0))
+			CC_on_commit(self);
+		else if (strnicmp(query, "ROLLBACK", 8) == 0)
+		{
+			/* 
+			   The method of ROLLBACK an original form ....
+			   ROLLBACK [ WORK | TRANSACTION ] TO [ SAVEPOINT ] savepoint_name
+			 */
+			if (PG_VERSION_LT(self, 8.0) || !(contains_token(query, " TO ")))
+				CC_on_abort(self, NO_TRANS);
+		}
+		else if (strnicmp(query, "ABORT", 5) == 0)
+			CC_on_abort(self, NO_TRANS);
+		else
+		{
+			if (PQcmdTuples(pgres)[0])
+				qres->recent_processed_row_count = atoi(PQcmdTuples(pgres));
+			else
+				qres->recent_processed_row_count = -1;
+			mylog("LIBPQ_execute_query: recent_processed_row_count = %i\n",qres->recent_processed_row_count);
+		}
 		mylog("The query was executed successfully and the query did not return any result \n");
 		PQclear(pgres);
 		return qres;
@@ -1857,6 +1847,16 @@ LIBPQ_execute_query(ConnectionClass *self,char *query)
 		return qres;
 	}
 
+	mylog("LIBPQ_execute_query: rest types ...\n");
+
+	if (PQcmdTuples(pgres)[0])
+		qres->recent_processed_row_count = atoi(PQcmdTuples(pgres));
+	else if (self->connInfo.drivers.use_declarefetch)
+		qres->recent_processed_row_count = -1;
+	else
+		qres->recent_processed_row_count = PQntuples(pgres);
+	mylog("LIBPQ_execute_query: recent_processed_row_count = %i\n",qres->recent_processed_row_count);
+
 	qres=CC_mapping(self,pgres,qres);
 	QR_set_command(qres, query);
 	PQclear(pgres);
@@ -1870,21 +1870,25 @@ LIBPQ_execute_query(ConnectionClass *self,char *query)
 QResultClass *
 CC_mapping(ConnectionClass *self, PGresult *pgres,QResultClass *qres)
 {
+	CSTR func = "CC_mapping";
 	int i=0,j=0;
 	TupleNode *node, *temp;
 	Oid typid;
 	int atttypmod,typlen;
 	int num_attributes = PQnfields(pgres);
 	int num_tuples = PQntuples(pgres);
-    ConnInfo   *ci = &(self->connInfo);
+	ConnInfo   *ci = &(self->connInfo);
 
+	mylog("%s: entering ...\n",func);
 	CI_set_num_fields(qres->fields, num_attributes);
+	mylog("%s: rows = %i, columns = %i\n",func,num_tuples,num_attributes);
 	for(i = 0 ; i < num_attributes ; i++)
 	{
+		mylog("%s: column = %i\n",func,i);
 		typid = PQftype(pgres,i);
 		atttypmod = PQfmod(pgres,i);
         
-        /* Setup the typmod */
+		/* Setup the typmod */
 		switch (typid)
 		{
 			case PG_TYPE_DATETIME:
@@ -1898,41 +1902,42 @@ CC_mapping(ConnectionClass *self, PGresult *pgres,QResultClass *qres)
 		if (atttypmod < 0)
 			atttypmod = -1;
         
-        /* Setup the typlen */
-        switch (typid)
-        {
-            case PG_TYPE_NUMERIC:
-                typlen = (atttypmod >> 16) & 0xffff;
-                break;
+		/* Setup the typlen */
+		switch (typid)
+		{
+			case PG_TYPE_NUMERIC:
+				typlen = (atttypmod >> 16) & 0xffff;
+				break;
             
-            case PG_TYPE_BPCHAR:
-            case PG_TYPE_VARCHAR:
-                typlen = atttypmod;
-                break;
+			case PG_TYPE_BPCHAR:
+			case PG_TYPE_VARCHAR:
+				typlen = atttypmod;
+				break;
         
-            default:
-                typlen = PQfsize(pgres,i);
-        }
+			default:
+		                typlen = PQfsize(pgres,i);
+        	}
         
-        /* 
-         * FIXME - The following is somewhat borked with regard to driver.unknownsizes
-         *         It works, but basically is not aware of different variable length types
-         *         (UNKNOWNS_AS_MAX), and UNKNOWNS_AS_LONGEST won't work because we don't
-         *         have data at this point 
-         */
-		if(typlen == -1)
-        {
-            switch (ci->drivers.unknown_sizes)
-            {
-                case UNKNOWNS_AS_DONTKNOW:
-				    typlen = SQL_NO_TOTAL;
-                    break;
+	        /* 
+	         * FIXME - The following is somewhat borked with regard to driver.unknownsizes
+	         *         It works, but basically is not aware of different variable length types
+	         *         (UNKNOWNS_AS_MAX), and UNKNOWNS_AS_LONGEST won't work because we don't
+	         *         have data at this point 
+	         */
+		if(typlen < 0)
+		{
+			switch (ci->drivers.unknown_sizes)
+			{
+				case UNKNOWNS_AS_DONTKNOW:
+					typlen = SQL_NO_TOTAL;
+					break;
                 
-			    default:
-				    typlen = ci->drivers.max_longvarchar_size;
-            }
-		}
+				default:
+					typlen = ( ci->drivers.text_as_longvarchar ? ci->drivers.max_longvarchar_size : ci->drivers.max_varchar_size );
+			}
+		}	
         
+		mylog("%s: set field info: name = %s, typ = %i, typlen = %i, attypmod = %i\n", func, PQfname(pgres,i), typid, (Int2)typlen, atttypmod);
 		CI_set_field_info(qres->fields, i, PQfname(pgres,i),
 			  typid, (Int2)typlen, atttypmod);
 	}
@@ -1970,9 +1975,15 @@ CC_mapping(ConnectionClass *self, PGresult *pgres,QResultClass *qres)
 				/* PQgetvalue returns an empty string even the data value is null. 
 				  * An additional checking is provided to set the null value */
 				if (PQgetisnull(pgres,i,j)) 
+				{
+					mylog("%s: fetch column = %s, value = NULL\n",func,PQfname(pgres,j));
 					set_tuplefield_null(&qres->manual_tuples->list_end->tuple[j]);
+				}
 				else
+				{
+					mylog("%s: fetch column = %s, value = %s\n",func,PQfname(pgres,j),PQgetvalue(pgres,i,j));
 					set_tuplefield_string(&qres->manual_tuples->list_end->tuple[j], PQgetvalue(pgres,i,j));
+				}
 			}
 
 	}
