@@ -96,9 +96,11 @@ Int2		sqlTypes[] = {
 	SQL_TINYINT,
 	SQL_VARBINARY,
 	SQL_VARCHAR,
+#ifdef  UNICODE_SUPPORT
 	SQL_WCHAR,
 	SQL_WVARCHAR,
 	SQL_WLONGVARCHAR,
+#endif
 	0
 };
 
@@ -126,14 +128,17 @@ sqltype_to_pgtype(StatementClass *stmt, SWORD fSqlType)
 			pgType = PG_TYPE_BPCHAR;
 			break;
 
+#ifdef  UNICODE_SUPPORT
 		case SQL_WCHAR:
 			pgType = PG_TYPE_BPCHAR;
 			break;
+#endif
 
 		case SQL_BIT:
 			pgType = ci->drivers.bools_as_char ? PG_TYPE_CHAR : PG_TYPE_BOOL;
 			break;
 
+        case SQL_DATE:
 		case SQL_TYPE_DATE:
 			pgType = PG_TYPE_DATE;
 			break;
@@ -167,9 +172,11 @@ sqltype_to_pgtype(StatementClass *stmt, SWORD fSqlType)
 			pgType = ci->drivers.text_as_longvarchar ? PG_TYPE_TEXT : PG_TYPE_VARCHAR;
 			break;
 
+#ifdef  UNICODE_SUPPORT
 		case SQL_WLONGVARCHAR:
 			pgType = ci->drivers.text_as_longvarchar ? PG_TYPE_TEXT : PG_TYPE_VARCHAR;
 			break;
+#endif
 
 		case SQL_REAL:
 			pgType = PG_TYPE_FLOAT4;
@@ -198,9 +205,11 @@ sqltype_to_pgtype(StatementClass *stmt, SWORD fSqlType)
 			pgType = PG_TYPE_VARCHAR;
 			break;
 
+#ifdef  UNICODE_SUPPORT
 		case SQL_WVARCHAR:
 			pgType = PG_TYPE_VARCHAR;
 			break;
+#endif
 
 		default:
 			pgType = 0;			/* ??? */
@@ -237,9 +246,12 @@ pgtype_to_concise_type(StatementClass *stmt, Int4 type, int col)
 		case PG_TYPE_CHAR2:
 		case PG_TYPE_CHAR4:
 		case PG_TYPE_CHAR8:
-		case PG_TYPE_NAME:
 			return SQL_CHAR;
 
+		case PG_TYPE_NAME:
+			return SQL_VARCHAR;
+
+#ifdef  UNICODE_SUPPORT
 		case PG_TYPE_BPCHAR:
 			if (col >= 0 &&
 			    getCharColumnSize(stmt, type, col, UNKNOWNS_AS_MAX) > ci->drivers.max_varchar_size)
@@ -256,6 +268,22 @@ pgtype_to_concise_type(StatementClass *stmt, Int4 type, int col)
 			return ci->drivers.text_as_longvarchar ? 
 				(conn->unicode ? SQL_WLONGVARCHAR : SQL_LONGVARCHAR) :
 				(conn->unicode ? SQL_WVARCHAR : SQL_VARCHAR);
+#else 	 
+		case PG_TYPE_BPCHAR: 	 
+			if (col >= 0 && 
+			    getCharColumnSize(stmt, type, col, UNKNOWNS_AS_MAX) > ci->drivers.max_varchar_size) 	 
+				return SQL_LONGVARCHAR; 	 
+			return SQL_CHAR; 	 
+  	 
+		case PG_TYPE_VARCHAR: 	 
+			if (col >= 0 && 	 
+			    getCharColumnSize(stmt, type, col, UNKNOWNS_AS_MAX) > ci->drivers.max_varchar_size) 	 
+				return SQL_LONGVARCHAR; 	 
+			return SQL_VARCHAR; 	 
+  	 
+		case PG_TYPE_TEXT: 	 
+			return ci->drivers.text_as_longvarchar ? SQL_LONGVARCHAR : SQL_VARCHAR; 	 
+#endif /* UNICODE_SUPPORT */
 
 		case PG_TYPE_BYTEA:
 			if (ci->bytea_as_longvarbinary)
@@ -406,12 +434,14 @@ pgtype_to_ctype(StatementClass *stmt, Int4 type)
 			return SQL_C_BINARY;
 		case PG_TYPE_LO_UNDEFINED:
 			return SQL_C_BINARY;
+#ifdef  UNICODE_SUPPORT
 		case PG_TYPE_BPCHAR:
 		case PG_TYPE_VARCHAR:
 		case PG_TYPE_TEXT:
 			if (conn->unicode && ! conn->ms_jet && ! stmt->manual_result)
 				return SQL_C_WCHAR;
 			return SQL_C_CHAR;
+#endif
 
 		default:
 			/* hack until permanent type is available */
@@ -549,7 +579,8 @@ getNumericDecimalDigits(StatementClass *stmt, Int4 type, int col)
 static Int4
 getNumericColumnSize(StatementClass *stmt, Int4 type, int col)
 {
-	Int4	atttypmod = -1, max_column_size = PG_NUMERIC_MAX_PRECISION + PG_NUMERIC_MAX_SCALE, default_column_size = 28;
+	Int4	atttypmod = -1;
+	Int4	default_column_size = 28;
 	QResultClass *result;
 	ColumnInfoClass *flds;
 
@@ -1396,12 +1427,14 @@ sqltype_to_default_ctype(const ConnectionClass *conn, Int2 sqltype)
 			return SQL_C_CHAR;
 		case SQL_BIGINT:
 			return ALLOWED_C_BIGINT;
+#ifdef  UNICODE_SUPPORT
 		case SQL_WCHAR:
 		case SQL_WVARCHAR:
 		case SQL_WLONGVARCHAR:
 			if (conn->ms_jet || ! conn->unicode)
 				return SQL_C_CHAR;
 			return SQL_C_WCHAR;
+#endif
 
 		case SQL_BIT:
 			return SQL_C_BIT;
@@ -1500,7 +1533,9 @@ ctype_length(Int2 ctype)
 
 		case SQL_C_BINARY:
 		case SQL_C_CHAR:
+#ifdef  UNICODE_SUPPORT
 		case SQL_C_WCHAR:
+#endif
 			return 0;
 
 		default:				/* should never happen */
