@@ -10,34 +10,11 @@
 #define __QRESULT_H__
 
 #include "psqlodbc.h"
-
 #include "connection.h"
-#include "socket.h"
 #include "columninfo.h"
 #include "tuplelist.h"
 #include "tuple.h"
 
-enum QueryResultCode_
-{
-	PGRES_EMPTY_QUERY = 0,
-	PGRES_COMMAND_OK,			/* a query command that doesn't return */
-	/* anything was executed properly by the backend */
-	PGRES_TUPLES_OK,			/* a query command that returns tuples */
-	/* was executed properly by the backend, PGresult */
-	/* contains the resulttuples */
-	PGRES_COPY_OUT,
-	PGRES_COPY_IN,
-	PGRES_BAD_RESPONSE,			/* an unexpected response was recv'd from
-								 * the backend */
-	PGRES_NONFATAL_ERROR,
-	PGRES_FATAL_ERROR,
-	PGRES_FIELDS_OK,			/* field information from a query was
-								 * successful */
-	PGRES_END_TUPLES,
-	PGRES_INTERNAL_ERROR
-};
-typedef enum QueryResultCode_ QueryResultCode;
-
 
 struct QResultClass_
 {
@@ -61,7 +38,7 @@ struct QResultClass_
 	int			rowset_size;
 	Int4			recent_processed_row_count;
 
-	QueryResultCode status;
+	ExecStatusType status;
 
 	char	   *message;
 	char	   *cursor;			/* The name of the cursor for select
@@ -78,21 +55,22 @@ struct QResultClass_
 	char		haskeyset;	/* this result contains keyset ? */
 	KeySet		*keyset;
 	UInt4		reload_count;
-	UInt2		rb_alloc;	/* count of allocated rollback info */	
-	UInt2		rb_count;	/* count of rollback info */	
-	Rollback	*rollback;	
-	UInt2		dl_alloc;	/* count of allocated deleted info */	
-	UInt2		dl_count;	/* count of deleted info */	
-	UInt4		*deleted;	
+	UInt2		rb_alloc;	/* count of allocated rollback info */
+	UInt2		rb_count;	/* count of rollback info */
+	Rollback	*rollback;
+	UInt2		dl_alloc;	/* count of allocated deleted info */
+	UInt2		dl_count;	/* count of deleted info */
+	UInt4		*deleted;
 };
 
 #define QR_get_fields(self)					(self->fields)
 
 
 /*	These functions are for retrieving data from the qresult */
+/* Retrieve results from manual_tuples since it has the results */
 #define QR_get_value_manual(self, tupleno, fieldno) (TL_get_fieldval(self->manual_tuples, tupleno, fieldno))
-#define QR_get_value_backend(self, fieldno)			(self->tupleField[fieldno].value)
-#define QR_get_value_backend_row(self, tupleno, fieldno) ((self->backend_tuples + (tupleno * self->num_fields))[fieldno].value)
+#define QR_get_value_backend(self,fieldno)			(TL_get_fieldval(self->manual_tuples,self->currTuple, fieldno))
+#define QR_get_value_backend_row(self, tupleno, fieldno) (TL_get_fieldval(self->manual_tuples, tupleno, fieldno))
 
 /*	These functions are used by both manual and backend results */
 #define QR_NumResultCols(self)		(CI_get_num_fields(self->fields))
@@ -113,6 +91,7 @@ struct QResultClass_
 #define QR_command_successful(self)	( !(self->status == PGRES_BAD_RESPONSE || self->status == PGRES_NONFATAL_ERROR || self->status == PGRES_FATAL_ERROR))
 #define QR_command_maybe_successful(self) ( !(self->status == PGRES_BAD_RESPONSE || self->status == PGRES_FATAL_ERROR))
 #define QR_command_nonfatal(self)	( self->status == PGRES_NONFATAL_ERROR)
+#define QR_command_fatal(self)	( self->status == PGRES_FATAL_ERROR)
 #define QR_end_tuples(self)		( self->status == PGRES_END_TUPLES)
 #define QR_set_status(self, condition)		( self->status = condition )
 #define QR_set_aborted(self, aborted_)		( self->aborted = aborted_)
