@@ -70,6 +70,9 @@ static struct
 	{
 		STMT_TYPE_LOCK, "LOCK"
 	},
+	{
+		STMT_TYPE_BEGIN, "BEGIN"
+	},
 	{
 		0, NULL
 	}
@@ -242,9 +245,11 @@ SC_Constructor(void)
 {
 	StatementClass *rv;
 
+	mylog("SC_Constructor: entering ...\n");
 	rv = (StatementClass *) malloc(sizeof(StatementClass));
 	if (rv)
 	{
+		mylog("SC_Constructor: created stmt = %u\n",rv);
 		rv->hdbc = NULL;		/* no connection associated yet */
 		rv->phstmt = NULL;
 		rv->result = NULL;
@@ -421,6 +426,16 @@ statement_type(const char *statement)
 		if (!strnicmp(statement, Statement_Type[i].s, strlen(Statement_Type[i].s)))
 			return Statement_Type[i].type;
 
+	/* determine START TRANSACTION */
+	if (!strnicmp(statement, "START", 5))
+	{
+		statement += 5;
+		/* ignore whitespace in query string */
+		while (*statement && isspace((UCHAR) *statement))
+			statement++;
+		if (!strnicmp(statement, "TRANSACTION", 11))
+			return STMT_TYPE_BEGIN;
+	}
 	return STMT_TYPE_OTHER;
 }
 
@@ -588,7 +603,8 @@ SC_recycle_statement(StatementClass *self)
 			 * transaction.
 			 */
 			conn = SC_get_conn(self);
-			if (!CC_is_in_autocommit(conn) && CC_is_in_trans(conn))
+			if (CC_is_in_trans(conn) && !CC_is_in_autocommit(conn) && 
+			    !CC_is_in_manual_trans(conn))
 			{
 				if (SC_is_pre_executable(self) && !conn->connInfo.disallow_premature)
 					CC_abort(conn);
@@ -974,6 +990,7 @@ SC_fetch(StatementClass *self)
 			if ((((self->currTuple + 1) % ci->drivers.fetch_max) >= QR_get_num_total_tuples(res)) &&
 				QR_get_num_total_tuples(res) < ci->drivers.fetch_max)
 			{
+				SC_no_fetchcursor(self);
 				self->currTuple = QR_get_num_total_tuples(res);
 				return SQL_NO_DATA_FOUND;
 			}
@@ -1144,7 +1161,7 @@ SC_execute(StatementClass *self)
 	QueryInfo	qi;
 	ConnInfo   *ci;
 	UDWORD		qflag = 0;
-	BOOL		auto_begin = FALSE, is_in_trans;
+	BOOL		is_in_trans;
 	int		func_cs_count = 0;
 
 
@@ -1171,17 +1188,16 @@ SC_execute(StatementClass *self)
 	is_in_trans = CC_is_in_trans(conn);
 	if (!self->internal && !is_in_trans &&
 		(SC_is_fetchcursor(self) ||
-		 (!CC_is_in_autocommit(conn) && self->statement_type != STMT_TYPE_OTHER)))
+		 (!CC_is_in_autocommit(conn) && self->statement_type != STMT_TYPE_BEGIN)))
 	{
 		mylog("   about to begin a transaction on statement = %u\n", self);
-		auto_begin = TRUE;
 		if (PG_VERSION_GE(conn, 7.1))
 			qflag |= GO_INTO_TRANSACTION;
-                else if (!CC_begin(conn))
-                {
+		else if (!CC_begin(conn))
+		{
 			SC_set_error(self, STMT_EXEC_ERROR, "Could not begin a transaction");
 			goto cleanup;
-                }
+		}
 	}
 
 	oldstatus = conn->status;
@@ -1285,8 +1301,10 @@ SC_execute(StatementClass *self)
 		/* issue "ABORT" when query aborted */
 		if (QR_get_aborted(res))
 		{
-			if (!self->internal)
+			if (!self->internal && CC_is_in_trans(conn) && CC_is_in_autocommit(conn) && !CC_is_in_manual_trans(conn))
 				CC_abort(conn);
+			else
+				QR_set_aborted(res, FALSE);
 		}
 		else
 		{
@@ -1332,7 +1350,7 @@ SC_execute(StatementClass *self)
 			SC_set_error(self, STMT_EXEC_ERROR, CC_get_errormsg(conn));
 		}
 
-		if (!self->internal)
+		if (!self->internal && CC_is_in_trans(conn) && CC_is_in_autocommit(conn) && !CC_is_in_manual_trans(conn))
 			CC_abort(conn);
 	}
 	if (!SC_get_Result(self))
@@ -1463,8 +1481,8 @@ SC_log_error(const char *func, const char *desc, const StatementClass *self)
 
 		rowsetSize = (7 == self->transition_status ? opts->size_of_rowset_odbc2 : opts->size_of_rowset);
 
-		qlog("STATEMENT ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__error_message));
-		mylog("STATEMENT ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__error_message));
+		qlog("STATEMENT ERROR: func=%s, desc='%s', errnum=%d, sqlstate=%s, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__sqlstate), nullcheck(self->__error_message));
+		mylog("STATEMENT ERROR: func=%s, desc='%s', errnum=%d, sqlstate=%s, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__sqlstate), nullcheck(self->__error_message));
 		qlog("                 ------------------------------------------------------------\n");
 		qlog("                 hdbc=%u, stmt=%u, result=%u\n", self->hdbc, self, res);
 		qlog("                 manual_result=%d, prepare=%d, internal=%d\n", self->manual_result, self->prepare, self->internal);
