@@ -96,7 +96,8 @@ enum
 	STMT_TYPE_GRANT,
 	STMT_TYPE_REVOKE,
 	STMT_TYPE_PROCCALL,
-	STMT_TYPE_LOCK
+	STMT_TYPE_LOCK,
+	STMT_TYPE_BEGIN
 };
 
 #define STMT_UPDATE(stmt)	(stmt->statement_type > STMT_TYPE_SELECT)
